@@ -1,11 +1,14 @@
 """
 Document Extend
 """
+import typing
 import warnings
-import elasticsearch
 
-from elasticsearch_dsl import Document, Q, connections, Search, Index
-from wisdoms_dapr import exceptions
+import elasticsearch
+from elasticsearch_dsl import Document, Index, Q, Search, connections, field
+from pydantic import BaseModel
+
+from wisdoms_dapr import exceptions, shares
 
 
 class ExtendDocument(Document):
@@ -118,7 +121,7 @@ class ExtendDocument(Document):
                             continue
                         # print('attr_names_list append string list:', attr_name)
                         attr_names_list.append('.'.join(attr_name))
-                        attr_name.clear()  # 清空列表
+                        attr_name.clear() # 清空列表
 
             _get_type_attr_from_dict(attr_obj, attr_name=attr_name)
             return attr_names_list
@@ -194,10 +197,10 @@ class ExtendDocument(Document):
 
         if 'unique' in extend_settings:
             # 获取unique属性列表
-            unique_attrs = (extend_settings['unique']
-                            if isinstance(extend_settings['unique'], list) or isinstance(extend_settings['unique'],
-                                                                                         tuple)
-                            else [extend_settings['unique']])
+            unique_attrs = (
+                extend_settings['unique'] if isinstance(extend_settings['unique'], list)
+                or isinstance(extend_settings['unique'], tuple) else [extend_settings['unique']]
+            )
 
             # 判断对象是否为关键字类型属性
             unique_attrs = self._get_unique_keyword_list(unique_attrs)
@@ -323,9 +326,13 @@ class ExtendDocument(Document):
             return doc
 
     @classmethod
-    def update_document(cls, id, data):
-        es = connections.get_connection()
-        es.update(index=cls.Index.name, doc_type='doc', id=id, body={'doc': data})
+    def update_document(cls, id, data, *, exclude_id: bool = True) -> 'ExtendDocument':
+        if exclude_id:
+            data.pop('id', None)
+
+        doc = cls.get_document_by_id(document_id=id, raise_exception=True)
+        doc.update(**data)
+        return doc
 
     @staticmethod
     def _search_item_to_dict(search_item):
@@ -409,6 +416,68 @@ class ExtendDocument(Document):
 
         cls._index.flush()
 
+    @classmethod
+    def create_or_update_document(cls, data: dict) -> typing.Tuple['ExtendDocument', bool]:
+        """创建或者更新文档
+        
+        当id存在时，通过id查找并更新对应的文档
+        当id不存在时，创建文档
+
+        :return {
+            data: return data dict,
+            created: bool, if created return True else return False
+        }
+        """
+
+        created = False
+        if 'id' in data and data['id']:
+            # 更新文档
+            doc = cls.get_document_by_id(document_id=data['id'], raise_exception=True)
+            data.pop('id')
+            res = doc.update(**data, is_check=True)
+        else:
+            # 创建文档
+            doc = cls(**data)
+            res = doc.save(is_check=True)
+            created = True
+
+        return res, created
+
+    @classmethod
+    def delete_documents(cls, ids: typing.Union[typing.List[str], str]):
+        """删除文档"""
+
+        if ids and not isinstance(ids, list):
+            ids = [ids]
+
+        query = Q('ids', **{'values': ids})
+        search = Search(index=cls.Index.name).query(query)
+        res = search.delete()
+        return res
+
+    def to_schema_dict(self, schema: typing.Optional[typing.Type[BaseModel]] = None, **schema_kwargs) -> dict:
+        """Convert to schema dict"""
+
+        data = self.to_id_dict()
+
+        if not schema:
+            return data
+
+        return schema(**data).dict(**schema_kwargs)
+
+    def to_id_dict(self) -> dict:
+        """Convert to dict include id in dict"""
+
+        data = self.to_dict()
+        try:
+            doc_id = self.meta.id
+        except AttributeError:
+            return data
+        else:
+            data['id'] = doc_id
+
+        return data
+
 
 def init_document_index(document_cls, connection=None, *, delete_on_conflict=False):
     """初始化文档索引
@@ -456,3 +525,189 @@ def init_document_index(document_cls, connection=None, *, delete_on_conflict=Fal
 
         document_cls.init()
         # print('document `{}` index created.'.format(document_index_name)
+
+
+# Elasticsearch and Meta Fields Mapping
+META_FIELD_ELASTICSEARCH_MAPPING = {
+    shares.AllMetaFieldTypes.BOOL.value: field.Boolean,
+    shares.AllMetaFieldTypes.STRING.value: field.Keyword,
+    shares.AllMetaFieldTypes.TEXT.value: field.Text,
+    shares.AllMetaFieldTypes.BINARY.value: field.Binary,
+    shares.AllMetaFieldTypes.INT.value: field.Integer,
+    shares.AllMetaFieldTypes.FLOAT.value: field.Float,
+    shares.AllMetaFieldTypes.OBJECT.value: field.Object,
+    shares.AllMetaFieldTypes.NESTED.value: field.Nested,
+    shares.AllMetaFieldTypes.DATE.value: field.Date,
+    shares.AllMetaFieldTypes.IP.value: field.Ip,
+    shares.AllMetaFieldTypes.GEO_POINT.value: field.GeoPoint,
+}
+META_FIELD_ELASTICSEARCH_TYPE_MAPPING = {k: v.name for k, v in META_FIELD_ELASTICSEARCH_MAPPING.items()}
+ELASTICSEARCH_META_FIELD_TYPE_MAPPING = dict(
+    zip(META_FIELD_ELASTICSEARCH_TYPE_MAPPING.values(), META_FIELD_ELASTICSEARCH_TYPE_MAPPING.keys())
+)
+# 内置全局变量
+MODEL_CREATED_TIME_FIELD_NAME = 'created_time'
+MODEL_MODIFIED_TIME_FIELD_NAME = 'modified_time'
+
+
+class SchemaDocument(Document):
+    """Schema Document"""
+
+    class Schema:
+        """
+        Schema Info
+
+        :attr meta: schema Meta
+        :attr fields: fields settings
+        """
+
+        pass
+
+    @classmethod
+    def get_all_model_fields(cls, *, include_type_obj: bool = False, include_schema: bool = False) -> dict:
+        """获取mapping下面的fields
+
+        :param include_type_obj: 是否包含字段类型
+        :param include_schema: 是否包含schema字段信息
+        """
+
+        def _get_all_model_fields(doc_class, include_type_obj: bool = False, include_schema: bool = False):
+
+            result = {}
+            properties = doc_class._doc_type.mapping.properties._params.get('properties')
+            if not properties:
+                return result
+
+            schema = None
+            if include_schema:
+                schema = getattr(doc_class, 'Schema', None)
+
+            for k, v in properties.items():
+                result[k] = {'type': v.name}
+
+                if include_type_obj:
+                    result[k]['type_obj'] = v
+
+                # 获取schema field
+                if schema:
+                    result[k]['schema'] = getattr(schema, 'fields', {}).get(k)
+
+                if v.name in [field.Object.name, field.Nested.name]:
+                    result[k]['fields'] = _get_all_model_fields(
+                        v._doc_class, include_type_obj=include_type_obj, include_schema=include_schema
+                    )
+
+            return result
+
+        # 添加创建时间字段
+        result = _get_all_model_fields(doc_class=cls, include_type_obj=include_type_obj, include_schema=include_schema)
+        if MODEL_CREATED_TIME_FIELD_NAME not in result:
+            result[MODEL_CREATED_TIME_FIELD_NAME] = {
+                'type': field.Date.name,
+                'type_obj': field.Date(),
+                'schema': {
+                    'dump_only': True
+                }
+            }
+        else:
+            if result[MODEL_CREATED_TIME_FIELD_NAME]['type'] != field.Date.name:
+                raise TypeError(f"model `{MODEL_CREATED_TIME_FIELD_NAME}` field must be date type.")
+
+        # 添加编辑时间字段
+        if MODEL_MODIFIED_TIME_FIELD_NAME not in result:
+            result[MODEL_MODIFIED_TIME_FIELD_NAME] = {
+                'type': field.Date.name,
+                'type_obj': field.Date(),
+                'schema': {
+                    'dump_only': True
+                }
+            }
+        else:
+            if result[MODEL_MODIFIED_TIME_FIELD_NAME]['type'] != field.Date.name:
+                raise TypeError(f"model `{MODEL_MODIFIED_TIME_FIELD_NAME}` field must be date type.")
+
+        return result
+
+    @classmethod
+    def get_model_metadata(cls) -> dict:
+        """导出模型元数据
+
+        数据结构同meta model和meta field model
+        """
+
+        def _get_fields_metadata(fields: dict, result_list: list, prefix: str = None):
+            """
+            获取字段元数据
+
+            :param fields: 所有模型的字段
+            :param result_list: 提取后的结构列表
+            :param prefix: 字段前缀
+            :return:
+            """
+
+            for field_name, item in fields.items():
+                if not item.get('type'):
+                    continue
+
+                meta_field_name = field_name if not prefix else str(prefix) + '.' + field_name
+                field_type = ELASTICSEARCH_META_FIELD_TYPE_MAPPING.get(item['type'])
+                if not field_type:
+                    raise exceptions.ServiceException(msg=f"elasticsearch {item['type']} not meta field mapping")
+
+                meta = {
+                    'model_name': cls.Index.name,
+                    'name': meta_field_name,
+                    'alias': "",
+                    'field_type': field_type,
+                    'order': 0,
+                    'searchable': False,
+                    'model_settings': {
+                                                           # TODO: model settings
+                    },
+                    'schema_settings': item.get('schema'),
+                    'is_valid': True,
+                    'description': None
+                }
+                result_list.append(meta)
+
+                if item['type'] in [field.Object.name, field.Nested.name] and item['fields']:
+                    _get_fields_metadata(fields=item['fields'], result_list=result_list, prefix=meta_field_name)
+
+        meta_model_fields = []
+        all_model_fields = cls.get_all_model_fields(include_type_obj=True, include_schema=True)
+        _get_fields_metadata(fields=all_model_fields, result_list=meta_model_fields)
+
+        metadata = {
+            'meta_model': {
+                'model_name': cls.Index.name,
+                'model_settings': {
+                    'elasticsearch_settings': {
+                        "index": {k: v
+                                  for k, v in cls.Index.__dict__.items() if not k.startswith('__')}
+                    }
+                },
+                'schema_settings': {
+                    'unknown': 'exclude'
+                },
+                'is_valid': True,
+                'description': cls.__doc__
+            },
+            'meta_model_fields': meta_model_fields
+        }
+
+        return metadata
+
+    @classmethod
+    def export_metadata(cls, init: bool = True):
+
+        # 初始化模型
+        if init:
+            cls.init()
+
+        # 获取模型元数据
+        metadata = cls.get_model_metadata()
+
+        # 发送至微服务保存
+        # logger.info(f"{cls.Index.name} model metadata: {metadata}")
+        # r = caller(service='common', method='import_model_metadata', raise_exception=True)(metadata)
+        # logger.info(f"export metadata result: {r}")
